from kagglesdk.kaggle_object import *
from kagglesdk.users.types.users_enums import UserAchievementTier
from typing import Optional

class UserAvatar(KaggleObject):
  r"""
  Attributes:
    display_name (str)
      Display name for the given user
    thumbnail_url (str)
      Thumbnail URL for the given user
    url (str)
      Profile URL for the given user
    profile_url (str)
      Alternate name for 'url', to aid in refactoring both are provided.
      TODO(http://b/402224065) remove once clients have migrated from this.
    user_name (str)
      User name for the given user
    tier (UserAchievementTier)
      Tier for the given user
    performance_tier (UserAchievementTier)
      Alternate name for `tier`, to aid in refactoring both are provided.
      TODO(http://b/402224065) remove once clients have migrated from this.
    id (int)
      ID for the given user
    user_id (int)
      Alternate name for `id`, to aid in refactoring both are provided.
      TODO(http://b/402224065) remove once clients have migrated from this.
    progression_opt_out (bool)
      True if the user is opted out of the progression system.
  """

  def __init__(self):
    self._display_name = None
    self._thumbnail_url = None
    self._url = None
    self._profile_url = None
    self._user_name = None
    self._tier = UserAchievementTier.NOVICE
    self._performance_tier = None
    self._id = 0
    self._user_id = None
    self._progression_opt_out = None
    self._freeze()

  @property
  def display_name(self) -> str:
    """Display name for the given user"""
    return self._display_name or ""

  @display_name.setter
  def display_name(self, display_name: Optional[str]):
    if display_name is None:
      del self.display_name
      return
    if not isinstance(display_name, str):
      raise TypeError('display_name must be of type str')
    self._display_name = display_name

  @property
  def thumbnail_url(self) -> str:
    """Thumbnail URL for the given user"""
    return self._thumbnail_url or ""

  @thumbnail_url.setter
  def thumbnail_url(self, thumbnail_url: Optional[str]):
    if thumbnail_url is None:
      del self.thumbnail_url
      return
    if not isinstance(thumbnail_url, str):
      raise TypeError('thumbnail_url must be of type str')
    self._thumbnail_url = thumbnail_url

  @property
  def url(self) -> str:
    """Profile URL for the given user"""
    return self._url or ""

  @url.setter
  def url(self, url: Optional[str]):
    if url is None:
      del self.url
      return
    if not isinstance(url, str):
      raise TypeError('url must be of type str')
    self._url = url

  @property
  def profile_url(self) -> str:
    r"""
    Alternate name for 'url', to aid in refactoring both are provided.
    TODO(http://b/402224065) remove once clients have migrated from this.
    """
    return self._profile_url or ""

  @profile_url.setter
  def profile_url(self, profile_url: Optional[str]):
    if profile_url is None:
      del self.profile_url
      return
    if not isinstance(profile_url, str):
      raise TypeError('profile_url must be of type str')
    self._profile_url = profile_url

  @property
  def user_name(self) -> str:
    """User name for the given user"""
    return self._user_name or ""

  @user_name.setter
  def user_name(self, user_name: Optional[str]):
    if user_name is None:
      del self.user_name
      return
    if not isinstance(user_name, str):
      raise TypeError('user_name must be of type str')
    self._user_name = user_name

  @property
  def tier(self) -> 'UserAchievementTier':
    """Tier for the given user"""
    return self._tier

  @tier.setter
  def tier(self, tier: 'UserAchievementTier'):
    if tier is None:
      del self.tier
      return
    if not isinstance(tier, UserAchievementTier):
      raise TypeError('tier must be of type UserAchievementTier')
    self._tier = tier

  @property
  def performance_tier(self) -> 'UserAchievementTier':
    r"""
    Alternate name for `tier`, to aid in refactoring both are provided.
    TODO(http://b/402224065) remove once clients have migrated from this.
    """
    return self._performance_tier or UserAchievementTier.NOVICE

  @performance_tier.setter
  def performance_tier(self, performance_tier: Optional['UserAchievementTier']):
    if performance_tier is None:
      del self.performance_tier
      return
    if not isinstance(performance_tier, UserAchievementTier):
      raise TypeError('performance_tier must be of type UserAchievementTier')
    self._performance_tier = performance_tier

  @property
  def id(self) -> int:
    """ID for the given user"""
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id

  @property
  def user_id(self) -> int:
    r"""
    Alternate name for `id`, to aid in refactoring both are provided.
    TODO(http://b/402224065) remove once clients have migrated from this.
    """
    return self._user_id or 0

  @user_id.setter
  def user_id(self, user_id: Optional[int]):
    if user_id is None:
      del self.user_id
      return
    if not isinstance(user_id, int):
      raise TypeError('user_id must be of type int')
    self._user_id = user_id

  @property
  def progression_opt_out(self) -> bool:
    """True if the user is opted out of the progression system."""
    return self._progression_opt_out or False

  @progression_opt_out.setter
  def progression_opt_out(self, progression_opt_out: Optional[bool]):
    if progression_opt_out is None:
      del self.progression_opt_out
      return
    if not isinstance(progression_opt_out, bool):
      raise TypeError('progression_opt_out must be of type bool')
    self._progression_opt_out = progression_opt_out


UserAvatar._fields = [
  FieldMetadata("displayName", "display_name", "_display_name", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("thumbnailUrl", "thumbnail_url", "_thumbnail_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("url", "url", "_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("profileUrl", "profile_url", "_profile_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("userName", "user_name", "_user_name", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("tier", "tier", "_tier", UserAchievementTier, UserAchievementTier.NOVICE, EnumSerializer()),
  FieldMetadata("performanceTier", "performance_tier", "_performance_tier", UserAchievementTier, None, EnumSerializer(), optional=True),
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
  FieldMetadata("userId", "user_id", "_user_id", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("progressionOptOut", "progression_opt_out", "_progression_opt_out", bool, None, PredefinedSerializer(), optional=True),
]




from kagglesdk.kaggle_object import *

class CreateInboxFileRequest(KaggleObject):
  r"""
  Attributes:
    virtual_directory (str)
      Directory name used for tagging the uploaded file.
    blob_file_token (str)
      Token representing the uploaded file.
  """

  def __init__(self):
    self._virtual_directory = ""
    self._blob_file_token = ""
    self._freeze()

  @property
  def virtual_directory(self) -> str:
    """Directory name used for tagging the uploaded file."""
    return self._virtual_directory

  @virtual_directory.setter
  def virtual_directory(self, virtual_directory: str):
    if virtual_directory is None:
      del self.virtual_directory
      return
    if not isinstance(virtual_directory, str):
      raise TypeError('virtual_directory must be of type str')
    self._virtual_directory = virtual_directory

  @property
  def blob_file_token(self) -> str:
    """Token representing the uploaded file."""
    return self._blob_file_token

  @blob_file_token.setter
  def blob_file_token(self, blob_file_token: str):
    if blob_file_token is None:
      del self.blob_file_token
      return
    if not isinstance(blob_file_token, str):
      raise TypeError('blob_file_token must be of type str')
    self._blob_file_token = blob_file_token

  def endpoint(self):
    path = '/api/v1/inbox/files/create'
    return path.format_map(self.to_field_map(self))


  @staticmethod
  def method():
    return 'POST'

  @staticmethod
  def body_fields():
    return '*'


class CreateInboxFileResponse(KaggleObject):
  r"""
  NOTE: This is sent to non-admins, so we're intentionally *NOT* sending back
  the full InboxFile (with its URL for a direct download).

  """

  pass

CreateInboxFileRequest._fields = [
  FieldMetadata("virtualDirectory", "virtual_directory", "_virtual_directory", str, "", PredefinedSerializer()),
  FieldMetadata("blobFileToken", "blob_file_token", "_blob_file_token", str, "", PredefinedSerializer()),
]

CreateInboxFileResponse._fields = []




import enum
from kagglesdk.kaggle_object import *
from typing import Optional

class LearnExerciseInteractionType(enum.Enum):
  LEARN_EXERCISE_INTERACTION_TYPE_UNSPECIFIED = 0
  CHECK = 1
  HINT = 2
  SOLUTION = 3

class LearnExerciseOutcomeType(enum.Enum):
  LEARN_EXERCISE_OUTCOME_TYPE_UNSPECIFIED = 0
  PASS = 1
  FAIL = 2
  EXCEPTION = 3
  UNATTEMPTED = 4

class LearnExerciseQuestionType(enum.Enum):
  LEARN_EXERCISE_QUESTION_TYPE_UNSPECIFIED = 0
  EQUALITY_CHECK_PROBLEM = 1
  CODING_PROBLEM = 2
  FUNCTION_PROBLEM = 3
  THOUGHT_EXPERIMENT = 4

class LearnNudgeType(enum.Enum):
  COURSE_COMPLETE_NO_BONUS_LESSONS = 0
  COURSE_COMPLETE_WITH_BONUS_LESSONS = 1
  COURSE_INCOMPLETE = 2
  DO_EXERCISE = 3
  DO_TUTORIAL = 4

class LearnNudge(KaggleObject):
  r"""
  Attributes:
    course_index (int)
    course_name (str)
    course_slug (str)
    next_item_name (str)
    next_item_url (str)
    next_item_type (LearnNudgeType)
  """

  def __init__(self):
    self._course_index = 0
    self._course_name = ""
    self._course_slug = ""
    self._next_item_name = ""
    self._next_item_url = ""
    self._next_item_type = LearnNudgeType.COURSE_COMPLETE_NO_BONUS_LESSONS
    self._freeze()

  @property
  def course_index(self) -> int:
    return self._course_index

  @course_index.setter
  def course_index(self, course_index: int):
    if course_index is None:
      del self.course_index
      return
    if not isinstance(course_index, int):
      raise TypeError('course_index must be of type int')
    self._course_index = course_index

  @property
  def course_name(self) -> str:
    return self._course_name

  @course_name.setter
  def course_name(self, course_name: str):
    if course_name is None:
      del self.course_name
      return
    if not isinstance(course_name, str):
      raise TypeError('course_name must be of type str')
    self._course_name = course_name

  @property
  def course_slug(self) -> str:
    return self._course_slug

  @course_slug.setter
  def course_slug(self, course_slug: str):
    if course_slug is None:
      del self.course_slug
      return
    if not isinstance(course_slug, str):
      raise TypeError('course_slug must be of type str')
    self._course_slug = course_slug

  @property
  def next_item_name(self) -> str:
    return self._next_item_name

  @next_item_name.setter
  def next_item_name(self, next_item_name: str):
    if next_item_name is None:
      del self.next_item_name
      return
    if not isinstance(next_item_name, str):
      raise TypeError('next_item_name must be of type str')
    self._next_item_name = next_item_name

  @property
  def next_item_url(self) -> str:
    return self._next_item_url

  @next_item_url.setter
  def next_item_url(self, next_item_url: str):
    if next_item_url is None:
      del self.next_item_url
      return
    if not isinstance(next_item_url, str):
      raise TypeError('next_item_url must be of type str')
    self._next_item_url = next_item_url

  @property
  def next_item_type(self) -> 'LearnNudgeType':
    return self._next_item_type

  @next_item_type.setter
  def next_item_type(self, next_item_type: 'LearnNudgeType'):
    if next_item_type is None:
      del self.next_item_type
      return
    if not isinstance(next_item_type, LearnNudgeType):
      raise TypeError('next_item_type must be of type LearnNudgeType')
    self._next_item_type = next_item_type


LearnNudge._fields = [
  FieldMetadata("courseIndex", "course_index", "_course_index", int, 0, PredefinedSerializer()),
  FieldMetadata("courseName", "course_name", "_course_name", str, "", PredefinedSerializer()),
  FieldMetadata("courseSlug", "course_slug", "_course_slug", str, "", PredefinedSerializer()),
  FieldMetadata("nextItemName", "next_item_name", "_next_item_name", str, "", PredefinedSerializer()),
  FieldMetadata("nextItemUrl", "next_item_url", "_next_item_url", str, "", PredefinedSerializer()),
  FieldMetadata("nextItemType", "next_item_type", "_next_item_type", LearnNudgeType, LearnNudgeType.COURSE_COMPLETE_NO_BONUS_LESSONS, EnumSerializer()),
]




import enum

class CompetitionListTab(enum.Enum):
  COMPETITION_LIST_TAB_GENERAL = 0
  """TODO(aip.dev/126): (-- api-linter: core::0126::unspecified=disabled --)"""
  COMPETITION_LIST_TAB_ENTERED = 1
  COMPETITION_LIST_TAB_COMMUNITY = 2
  COMPETITION_LIST_TAB_HOSTED = 3
  COMPETITION_LIST_TAB_UNLAUNCHED = 4
  COMPETITION_LIST_TAB_UNLAUNCHED_COMMUNITY = 5
  COMPETITION_LIST_TAB_EVERYTHING = 6

class CompetitionSortBy(enum.Enum):
  COMPETITION_SORT_BY_GROUPED = 0
  """TODO(aip.dev/126): (-- api-linter: core::0126::unspecified=disabled --)"""
  COMPETITION_SORT_BY_BEST = 1
  COMPETITION_SORT_BY_PRIZE = 2
  COMPETITION_SORT_BY_EARLIEST_DEADLINE = 3
  COMPETITION_SORT_BY_LATEST_DEADLINE = 4
  COMPETITION_SORT_BY_NUMBER_OF_TEAMS = 5
  COMPETITION_SORT_BY_RELEVANCE = 6
  COMPETITION_SORT_BY_RECENTLY_CREATED = 7

class HostSegment(enum.Enum):
  r"""
  NOTE: Keep in Sync with Kaggle.Entities.HostSegment until migrated! Also keep
  the comment in
  competition_service.ListCompetitionsRequest.Selector.host_segment_id_filter
  up to date
  """
  HOST_SEGMENT_UNSPECIFIED = 0
  HOST_SEGMENT_FEATURED = 1
  HOST_SEGMENT_GETTING_STARTED = 5
  HOST_SEGMENT_MASTERS = 6
  HOST_SEGMENT_PLAYGROUND = 8
  HOST_SEGMENT_RECRUITMENT = 3
  HOST_SEGMENT_RESEARCH = 2
  HOST_SEGMENT_COMMUNITY = 10
  HOST_SEGMENT_ANALYTICS = 11

class SubmissionGroup(enum.Enum):
  SUBMISSION_GROUP_ALL = 0
  """TODO(aip.dev/126): (-- api-linter: core::0126::unspecified=disabled --)"""
  SUBMISSION_GROUP_SUCCESSFUL = 1
  SUBMISSION_GROUP_SELECTED = 2

class SubmissionSortBy(enum.Enum):
  SUBMISSION_SORT_BY_DATE = 0
  """TODO(aip.dev/126): (-- api-linter: core::0126::unspecified=disabled --)"""
  SUBMISSION_SORT_BY_NAME = 1
  SUBMISSION_SORT_BY_PRIVATE_SCORE = 2
  SUBMISSION_SORT_BY_PUBLIC_SCORE = 3




from datetime import timedelta
import enum
from kagglesdk.kaggle_object import *
from kagglesdk.security.types.authentication import AuthorizationScope
from typing import Optional, List

class ApiVersion(enum.Enum):
  API_VERSION_UNSPECIFIED = 0
  API_VERSION_V1 = 1
  """Publicly available ('/api/v1' endpoints)."""
  API_VERSION_V2 = 2
  """Experimental, admin-only, internal ('/api/i' endpoints)."""

class ExpireApiTokenRequest(KaggleObject):
  r"""
  Attributes:
    token_id (int)
      Id of the token.
    token (str)
      Value of the token.
    reason (str)
      Reason why this token is expired.
  """

  def __init__(self):
    self._token_id = None
    self._token = None
    self._reason = None
    self._freeze()

  @property
  def token_id(self) -> int:
    """Id of the token."""
    return self._token_id or 0

  @token_id.setter
  def token_id(self, token_id: int):
    if token_id is None:
      del self.token_id
      return
    if not isinstance(token_id, int):
      raise TypeError('token_id must be of type int')
    del self.token
    self._token_id = token_id

  @property
  def token(self) -> str:
    """Value of the token."""
    return self._token or ""

  @token.setter
  def token(self, token: str):
    if token is None:
      del self.token
      return
    if not isinstance(token, str):
      raise TypeError('token must be of type str')
    del self.token_id
    self._token = token

  @property
  def reason(self) -> str:
    """Reason why this token is expired."""
    return self._reason or ""

  @reason.setter
  def reason(self, reason: Optional[str]):
    if reason is None:
      del self.reason
      return
    if not isinstance(reason, str):
      raise TypeError('reason must be of type str')
    self._reason = reason

  def endpoint(self):
    path = '/api/v1/tokens/revoke'
    return path.format_map(self.to_field_map(self))


  @staticmethod
  def method():
    return 'POST'

  @staticmethod
  def body_fields():
    return '*'


class GenerateAccessTokenRequest(KaggleObject):
  r"""
  Attributes:
    refresh_token (str)
      Refresh token used to generate a short-lived restricted access token.
      If not specified current user credentials will be used to generate an
      unrestricted access token.
    api_version (ApiVersion)
      Version of the Kaggle API for which this token will be used.
    expiration_duration (timedelta)
      Token expiration.
    authorization_context (AuthorizationContext)
      On which context (such as a Kaggle notebook) this token can be used.
    authorization_scopes (AuthorizationScope)
      Set of scopes to further restrict the token. If 'refresh_token' is
      specified, these should be a subset of the scopes allowed by the
      'refresh_token'.
  """

  def __init__(self):
    self._refresh_token = None
    self._api_version = ApiVersion.API_VERSION_UNSPECIFIED
    self._expiration_duration = None
    self._authorization_context = None
    self._authorization_scopes = []
    self._freeze()

  @property
  def refresh_token(self) -> str:
    r"""
    Refresh token used to generate a short-lived restricted access token.
    If not specified current user credentials will be used to generate an
    unrestricted access token.
    """
    return self._refresh_token or ""

  @refresh_token.setter
  def refresh_token(self, refresh_token: Optional[str]):
    if refresh_token is None:
      del self.refresh_token
      return
    if not isinstance(refresh_token, str):
      raise TypeError('refresh_token must be of type str')
    self._refresh_token = refresh_token

  @property
  def api_version(self) -> 'ApiVersion':
    """Version of the Kaggle API for which this token will be used."""
    return self._api_version

  @api_version.setter
  def api_version(self, api_version: 'ApiVersion'):
    if api_version is None:
      del self.api_version
      return
    if not isinstance(api_version, ApiVersion):
      raise TypeError('api_version must be of type ApiVersion')
    self._api_version = api_version

  @property
  def expiration_duration(self) -> timedelta:
    """Token expiration."""
    return self._expiration_duration

  @expiration_duration.setter
  def expiration_duration(self, expiration_duration: timedelta):
    if expiration_duration is None:
      del self.expiration_duration
      return
    if not isinstance(expiration_duration, timedelta):
      raise TypeError('expiration_duration must be of type timedelta')
    self._expiration_duration = expiration_duration

  @property
  def authorization_context(self) -> Optional['AuthorizationContext']:
    """On which context (such as a Kaggle notebook) this token can be used."""
    return self._authorization_context

  @authorization_context.setter
  def authorization_context(self, authorization_context: Optional['AuthorizationContext']):
    if authorization_context is None:
      del self.authorization_context
      return
    if not isinstance(authorization_context, AuthorizationContext):
      raise TypeError('authorization_context must be of type AuthorizationContext')
    self._authorization_context = authorization_context

  @property
  def authorization_scopes(self) -> Optional[List[Optional['AuthorizationScope']]]:
    r"""
    Set of scopes to further restrict the token. If 'refresh_token' is
    specified, these should be a subset of the scopes allowed by the
    'refresh_token'.
    """
    return self._authorization_scopes

  @authorization_scopes.setter
  def authorization_scopes(self, authorization_scopes: Optional[List[Optional['AuthorizationScope']]]):
    if authorization_scopes is None:
      del self.authorization_scopes
      return
    if not isinstance(authorization_scopes, list):
      raise TypeError('authorization_scopes must be of type list')
    if not all([isinstance(t, AuthorizationScope) for t in authorization_scopes]):
      raise TypeError('authorization_scopes must contain only items of type AuthorizationScope')
    self._authorization_scopes = authorization_scopes

  def endpoint(self):
    path = '/api/v1/access-tokens/generate'
    return path.format_map(self.to_field_map(self))


  @staticmethod
  def method():
    return 'POST'

  @staticmethod
  def body_fields():
    return '*'


class GenerateAccessTokenResponse(KaggleObject):
  r"""
  Attributes:
    token (str)
    expires_in (int)
  """

  def __init__(self):
    self._token = ""
    self._expires_in = 0
    self._freeze()

  @property
  def token(self) -> str:
    return self._token

  @token.setter
  def token(self, token: str):
    if token is None:
      del self.token
      return
    if not isinstance(token, str):
      raise TypeError('token must be of type str')
    self._token = token

  @property
  def expires_in(self) -> int:
    return self._expires_in

  @expires_in.setter
  def expires_in(self, expires_in: int):
    if expires_in is None:
      del self.expires_in
      return
    if not isinstance(expires_in, int):
      raise TypeError('expires_in must be of type int')
    self._expires_in = expires_in

  @property
  def expiresIn(self):
    return self.expires_in


class AuthorizationContext(KaggleObject):
  r"""
  Attributes:
    kernel_session_id (int)
      If set, access token is restricted to be used only from the specified
      notebook session.
  """

  def __init__(self):
    self._kernel_session_id = None
    self._freeze()

  @property
  def kernel_session_id(self) -> int:
    r"""
    If set, access token is restricted to be used only from the specified
    notebook session.
    """
    return self._kernel_session_id or 0

  @kernel_session_id.setter
  def kernel_session_id(self, kernel_session_id: Optional[int]):
    if kernel_session_id is None:
      del self.kernel_session_id
      return
    if not isinstance(kernel_session_id, int):
      raise TypeError('kernel_session_id must be of type int')
    self._kernel_session_id = kernel_session_id


ExpireApiTokenRequest._fields = [
  FieldMetadata("tokenId", "token_id", "_token_id", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("token", "token", "_token", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("reason", "reason", "_reason", str, None, PredefinedSerializer(), optional=True),
]

GenerateAccessTokenRequest._fields = [
  FieldMetadata("refreshToken", "refresh_token", "_refresh_token", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("apiVersion", "api_version", "_api_version", ApiVersion, ApiVersion.API_VERSION_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("expirationDuration", "expiration_duration", "_expiration_duration", timedelta, None, TimeDeltaSerializer()),
  FieldMetadata("authorizationContext", "authorization_context", "_authorization_context", AuthorizationContext, None, KaggleObjectSerializer()),
  FieldMetadata("authorizationScopes", "authorization_scopes", "_authorization_scopes", AuthorizationScope, [], ListSerializer(KaggleObjectSerializer())),
]

GenerateAccessTokenResponse._fields = [
  FieldMetadata("token", "token", "_token", str, "", PredefinedSerializer()),
  FieldMetadata("expiresIn", "expires_in", "_expires_in", int, 0, PredefinedSerializer()),
]

AuthorizationContext._fields = [
  FieldMetadata("kernelSessionId", "kernel_session_id", "_kernel_session_id", int, None, PredefinedSerializer(), optional=True),
]




import json
import re
from datetime import datetime, timedelta
from google.protobuf.field_mask_pb2 import FieldMask


class ObjectSerializer(object):

    def __init__(self, to_dict_value, from_dict_value):
        self.to_dict_value = to_dict_value
        self.from_dict_value = from_dict_value


class PredefinedSerializer(ObjectSerializer):

    def __init__(self):
        """Predefined objects such as int, float etc are serialized/deserialized directly."""
        ObjectSerializer.__init__(self, lambda cls, v, _: v, lambda cls, v: v)


# Adapted from https://stackoverflow.com/questions/1175208/elegant-python-function-to-convert-camelcase-to-snake-case
_pascal_to_upper_snake_case_regex = re.compile("((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))")


def _pascal_case_to_upper_snake_case(string):
    return _pascal_to_upper_snake_case_regex.sub(r"_\1", string).upper()


def _convert(camel_input):
    words = re.findall(r"[A-Z]?[a-z]+|[A-Z]{2,}(?=[A-Z][a-z]|\d|\W|$)|\d+", camel_input)
    return "_".join(map(str.lower, words))


class EnumSerializer(ObjectSerializer):

    def __init__(self):
        """
        Enum objects are serialized using their ".name" field and deserialized by indexing the string in the Enum type.
        Example:
        class Foo(Enum):
           TEST = 1
        foo = Foo.TEST
        foo.name # => returns "TEST"
        Foo["TEST"] # => returns Foo.TEST enum value.
        """
        ObjectSerializer.__init__(
            self,
            lambda cls, v, _: EnumSerializer._to_str(cls, v),
            lambda cls, v: EnumSerializer._from_str(cls, v),
        )

    @staticmethod
    def _to_str(cls, v):
        # "v" corresponds to an enum instance: Example foo or Foo.Test above.
        # "cls" corresponds to the enum type Foo above.
        return v.name

    @staticmethod
    def _from_str(cls, v):
        # "v" corresponds to enum string: Example "TEST" above.
        # "cls" corresponds to the enum type Foo above.
        try:
            return cls[v]
        except KeyError:
            dct = vars(cls)
            n = v.lower()
            nn = _convert(v).lower()
            enum_prefix = _pascal_case_to_upper_snake_case(cls.__name__).lower()
            for key in dct.keys():
                k = key.lower()
                if k == n:
                    return dct[key]
                if k.startswith(enum_prefix) and k.endswith(n) or k.endswith(nn):
                    return dct[key]
            raise


class ListSerializer(ObjectSerializer):

    def __init__(self, item_serializer: ObjectSerializer):
        """
        Lists are serialized based on the type they contain. Since objects are generated from proto files, a list always
        contains objects of the same type, which is serialized/deserialized using "item_serializer".
        """
        ObjectSerializer.__init__(
            self,
            lambda cls, l, ignore_defaults: [item_serializer.to_dict_value(cls, v, ignore_defaults) for v in l],
            lambda cls, l: [item_serializer.from_dict_value(cls, v) for v in l],
        )


class MapSerializer(ObjectSerializer):

    def __init__(self, item_serializer: ObjectSerializer):
        """
        Maps are serialized based on type of their values. Since maps keys are always predefined types, we don't need a
        serializer for them.
        """
        ObjectSerializer.__init__(
            self,
            lambda cls, d, ignore_defaults: {
                k: item_serializer.to_dict_value(cls, v, ignore_defaults) for k, v in d.items()
            },
            lambda cls, d: {k: item_serializer.from_dict_value(cls, v) for k, v in d.items()},
        )


class DateTimeSerializer(ObjectSerializer):

    def __init__(self):
        """Date times are serialized/deserialized as a string in iso format"""
        ObjectSerializer.__init__(
            self,
            lambda cls, dt, _: DateTimeSerializer._to_str(dt),
            lambda _, v: DateTimeSerializer._from_str(v),
        )

    @staticmethod
    def _to_str(dt):
        return dt.isoformat(timespec="milliseconds") + "Z"

    @staticmethod
    def _from_str(v):
        v = v.rstrip("Z")
        fields = v.rsplit(".", maxsplit=1)
        if len(fields) == 1:
            return datetime.fromisoformat(v)
        (dt, nanos) = fields
        millis = nanos[:3]
        try:
            return datetime.fromisoformat(f"{dt}.{millis}")
        except ValueError:
            return datetime.fromisoformat(dt)  # Python 3.9, 3.10


class TimeDeltaSerializer(ObjectSerializer):
    # Scaling factors idea from https://github.com/protocolbuffers/protobuf/blob/master/csharp/src/Google.Protobuf/JsonParser.cs
    SUBSECOND_SCALING_FACTORS = [
        0,
        100_000_000,
        10_000_000,
        1_000_000,
        100_000,
        10_000,
        1_000,
        100,
        10,
        1,
    ]

    def __init__(self):
        """Time deltas are serialized/deserialized as a string in "<seconds>.<nanoseconds>s" format. Example: 151.500s"""
        ObjectSerializer.__init__(
            self,
            lambda cls, t, _: TimeDeltaSerializer._to_dict_value(t),
            lambda cls, v: TimeDeltaSerializer._from_dict_value(v),
        )

    @staticmethod
    def _to_dict_value(delta):
        seconds = delta.seconds
        nanos_str = TimeDeltaSerializer._nanos_to_str(delta.microseconds * 1000)
        if nanos_str is None:
            return "{}s".format(seconds)
        return "{}.{}s".format(seconds, nanos_str)

    @staticmethod
    def _nanos_to_str(nanos):
        if nanos == 0:
            return None
        if nanos < 0:
            nanos *= -1
        if nanos % 1000000 == 0:
            return "{}".format(nanos / 1000000)
        elif nanos % 1000 == 0:
            return "{}".format(nanos / 1000)
        else:
            return "{}".format(nanos)

    @staticmethod
    def _from_dict_value(value):
        (seconds, nanosRaw) = value.rstrip("s").split(".")
        nanos = int(nanosRaw) * TimeDeltaSerializer.SUBSECOND_SCALING_FACTORS[len(nanosRaw)]
        return timedelta(seconds=int(seconds), microseconds=int(int(nanos) / 1000))


class FieldMaskSerializer(ObjectSerializer):

    def __init__(self):
        """Field masks are serialized/deserialized as a string that contains a list of paths with a comma delimiter"""
        ObjectSerializer.__init__(
            self,
            lambda cls, m, _: m.ToJsonString(),
            lambda cls, v: FieldMaskSerializer._from_joined_paths(v),
        )

    @staticmethod
    def _from_joined_paths(joined_paths):
        mask = FieldMask()
        mask.FromJsonString(joined_paths)
        return mask


class KaggleObjectSerializer(ObjectSerializer):

    def __init__(self):
        """
        Kaggle objects (i.e., proto-generated types that inherit from KaggleObject) have custom "to_dict" and "from_dict"
        methods that serialize/deserialize them to/from dictionaries.
        """
        ObjectSerializer.__init__(
            self,
            # "v" is an instance of a KaggleObject. For example: "req = ListCompetitionsRequest()".
            # So "req.to_dict()" returns a dictionary with keys as json field names. Example:
            # '{"pageSize": 10, "page": 2}'
            lambda cls, v, ignore_defaults: cls.to_dict(v, ignore_defaults),
            # "cls" is the type of a KaggleObject. For example: ListCompetitionsRequest. All
            # generated Kaggle objects have "from_dict" class method that takes a dict to create a
            # new instance of the object. See "KaggleObject" class definition below.
            lambda cls, v: cls.from_dict(v),
        )


class FieldMetadata(object):

    def __init__(
        self,
        json_name,
        field_name,
        private_field_name,
        field_type,
        default_value,
        serializer,
        optional=False,
    ):
        self.json_name = json_name
        self.field_name = field_name
        self.private_field_name = private_field_name
        self.field_type = field_type
        self.default_value = default_value
        self.serializer = serializer
        self.optional = optional

    def get_as_dict_item(self, instance, ignore_defaults=True):
        value = getattr(instance, self.private_field_name)
        if ignore_defaults and value == self.default_value:
            return None
        if value is None:
            return None
        return self.serializer.to_dict_value(self.field_type, value, ignore_defaults)

    def set_from_dict(self, instance, json_dict):
        if self.json_name not in json_dict:
            return  # Ignore unknown fields
        value = json_dict[self.json_name]
        if value == self.default_value:
            return  # Ignore default values
        try:
            setattr(
                instance,
                self.private_field_name,
                self.serializer.from_dict_value(self.field_type, value),
            )
        except Exception as e:
            raise


class KaggleObject(object):

    def endpoint(self):
        raise "Error: endpoint must be defined by the request object"

    @staticmethod
    def endpoint_path():
        return None

    @staticmethod
    def body_fields():
        return None

    @classmethod
    def prepare_from(cls, http_response):
        return cls.from_json(http_response.text)

    @staticmethod
    def method():
        return "GET"

    def _freeze(self):
        self._is_frozen = True

    def __setattr__(self, key, value):
        if hasattr(self, "_is_frozen") and not hasattr(self, key):
            raise AttributeError(f"Unknown field for {self.__class__.__name__}: {key}")
        object.__setattr__(self, key, value)

    def to_dict(self, ignore_defaults=True):
        kv_pairs = [(field.json_name, field.get_as_dict_item(self, ignore_defaults)) for field in self._fields]
        return {k: v for (k, v) in kv_pairs if not ignore_defaults or v is not None}

    def to_field_map(self, ignore_defaults=True):
        kv_pairs = [(field.field_name, field.get_as_dict_item(self, ignore_defaults)) for field in self._fields]
        return {k: v for (k, v) in kv_pairs if not ignore_defaults or v is not None}

    @classmethod
    def from_dict(cls, json_dict):
        instance = cls()
        for field in cls._fields:
            field.set_from_dict(instance, json_dict)
        return instance

    @classmethod
    def from_json(cls, json_str):
        return cls.from_dict(json.loads(json_str))

    def to_json(self, ignore_defaults=True):
        return json.dumps(self.to_dict(ignore_defaults))

    def __str__(self):
        return self.to_json(ignore_defaults=False)

    def __repr__(self):
        return self.to_json(ignore_defaults=False)

    def __contains__(self, field_name):
        try:
            field = self._get_field(field_name)
        except ValueError:
            return False
        value = getattr(self, field.private_field_name)
        if field.optional:
            return value is not None
        else:
            return value != field.default_value

    def __delattr__(self, field_name):
        field = self._get_field(field_name)
        setattr(self, field.private_field_name, field.default_value)

    def _get_field(self, field_name):
        field = next((f for f in self._fields if f.field_name == field_name), None)
        if field is None:
            raise ValueError(f'Protocol message {self.__class__.__name__} has no "{field_name}" field.')
        return field



from datetime import datetime
from kagglesdk.datasets.types.dataset_enums import DatabundleVersionStatus, DatabundleVersionType
from kagglesdk.datasets.types.dataset_service import DatabundleVersionCreationStatus
from kagglesdk.kaggle_object import *
from kagglesdk.licenses.types.licenses_types import License
from kagglesdk.models.types.model_enums import ModelFramework, ModelInstanceType, ModelVersionLinkType, SigstoreState
from kagglesdk.users.types.users_enums import UserAchievementTier
from typing import Optional, List

class BaseModelInstanceInformation(KaggleObject):
  r"""
  Attributes:
    id (int)
    owner (Owner)
    model_slug (str)
    instance_slug (str)
    framework (ModelFramework)
  """

  def __init__(self):
    self._id = 0
    self._owner = None
    self._model_slug = ""
    self._instance_slug = ""
    self._framework = ModelFramework.MODEL_FRAMEWORK_UNSPECIFIED
    self._freeze()

  @property
  def id(self) -> int:
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id

  @property
  def owner(self) -> Optional['Owner']:
    return self._owner

  @owner.setter
  def owner(self, owner: Optional['Owner']):
    if owner is None:
      del self.owner
      return
    if not isinstance(owner, Owner):
      raise TypeError('owner must be of type Owner')
    self._owner = owner

  @property
  def model_slug(self) -> str:
    return self._model_slug

  @model_slug.setter
  def model_slug(self, model_slug: str):
    if model_slug is None:
      del self.model_slug
      return
    if not isinstance(model_slug, str):
      raise TypeError('model_slug must be of type str')
    self._model_slug = model_slug

  @property
  def instance_slug(self) -> str:
    return self._instance_slug

  @instance_slug.setter
  def instance_slug(self, instance_slug: str):
    if instance_slug is None:
      del self.instance_slug
      return
    if not isinstance(instance_slug, str):
      raise TypeError('instance_slug must be of type str')
    self._instance_slug = instance_slug

  @property
  def framework(self) -> 'ModelFramework':
    return self._framework

  @framework.setter
  def framework(self, framework: 'ModelFramework'):
    if framework is None:
      del self.framework
      return
    if not isinstance(framework, ModelFramework):
      raise TypeError('framework must be of type ModelFramework')
    self._framework = framework


class ModelInstance(KaggleObject):
  r"""
  Attributes:
    id (int)
    owner_slug (str)
    model_slug (str)
    model_id (int)
    slug (str)
    version_id (int)
    fine_tunable (bool)
    overview (str)
    usage (str)
    rendered_usage (str)
    text_representation (str)
    source_url (str)
    version_number (int)
    framework (ModelFramework)
      TODO(http://b/253694274): Add support for variation tables when that's
      finished taking shape
    version_notes (str)
    download_url (str)
    databundle_id (int)
    databundle_version_id (int)
    databundle_version_type (DatabundleVersionType)
    firestore_path (str)
    status (DatabundleVersionStatus)
    creation_status (DatabundleVersionCreationStatus)
    error_message (str)
    last_version_id (int)
      Version ID associated with the most up-to-date version of the ModelInstance
    source_organization (Owner)
    training_data (str)
    metrics (str)
    license_post (LicensePost)
    license (License)
    can_use (bool)
    uncompressed_storage_uri (str)
    model_instance_type (ModelInstanceType)
    base_model_instance_id (int)
    base_model_instance_information (BaseModelInstanceInformation)
    external_base_model_url (str)
    download_summary (ModelInstanceDownloadSummary)
    total_uncompressed_bytes (int)
    sigstore_state (SigstoreState)
    created_by_kernel_id (int)
    creator_user_id (int)
    attestation_kernel_url (str)
  """

  def __init__(self):
    self._id = 0
    self._owner_slug = ""
    self._model_slug = ""
    self._model_id = 0
    self._slug = ""
    self._version_id = 0
    self._fine_tunable = None
    self._overview = ""
    self._usage = ""
    self._rendered_usage = ""
    self._text_representation = None
    self._source_url = None
    self._version_number = 0
    self._framework = ModelFramework.MODEL_FRAMEWORK_UNSPECIFIED
    self._version_notes = ""
    self._download_url = None
    self._databundle_id = 0
    self._databundle_version_id = 0
    self._databundle_version_type = DatabundleVersionType.DATABUNDLE_VERSION_TYPE_UNSPECIFIED
    self._firestore_path = ""
    self._status = DatabundleVersionStatus.NOT_YET_PERSISTED
    self._creation_status = None
    self._error_message = None
    self._last_version_id = None
    self._source_organization = None
    self._training_data = []
    self._metrics = None
    self._license_post = None
    self._license = None
    self._can_use = None
    self._uncompressed_storage_uri = None
    self._model_instance_type = None
    self._base_model_instance_id = None
    self._base_model_instance_information = None
    self._external_base_model_url = None
    self._download_summary = None
    self._total_uncompressed_bytes = None
    self._sigstore_state = None
    self._created_by_kernel_id = None
    self._creator_user_id = None
    self._attestation_kernel_url = None
    self._freeze()

  @property
  def id(self) -> int:
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id

  @property
  def owner_slug(self) -> str:
    return self._owner_slug

  @owner_slug.setter
  def owner_slug(self, owner_slug: str):
    if owner_slug is None:
      del self.owner_slug
      return
    if not isinstance(owner_slug, str):
      raise TypeError('owner_slug must be of type str')
    self._owner_slug = owner_slug

  @property
  def model_slug(self) -> str:
    return self._model_slug

  @model_slug.setter
  def model_slug(self, model_slug: str):
    if model_slug is None:
      del self.model_slug
      return
    if not isinstance(model_slug, str):
      raise TypeError('model_slug must be of type str')
    self._model_slug = model_slug

  @property
  def model_id(self) -> int:
    return self._model_id

  @model_id.setter
  def model_id(self, model_id: int):
    if model_id is None:
      del self.model_id
      return
    if not isinstance(model_id, int):
      raise TypeError('model_id must be of type int')
    self._model_id = model_id

  @property
  def slug(self) -> str:
    return self._slug

  @slug.setter
  def slug(self, slug: str):
    if slug is None:
      del self.slug
      return
    if not isinstance(slug, str):
      raise TypeError('slug must be of type str')
    self._slug = slug

  @property
  def version_id(self) -> int:
    return self._version_id

  @version_id.setter
  def version_id(self, version_id: int):
    if version_id is None:
      del self.version_id
      return
    if not isinstance(version_id, int):
      raise TypeError('version_id must be of type int')
    self._version_id = version_id

  @property
  def fine_tunable(self) -> bool:
    return self._fine_tunable or False

  @fine_tunable.setter
  def fine_tunable(self, fine_tunable: Optional[bool]):
    if fine_tunable is None:
      del self.fine_tunable
      return
    if not isinstance(fine_tunable, bool):
      raise TypeError('fine_tunable must be of type bool')
    self._fine_tunable = fine_tunable

  @property
  def overview(self) -> str:
    return self._overview

  @overview.setter
  def overview(self, overview: str):
    if overview is None:
      del self.overview
      return
    if not isinstance(overview, str):
      raise TypeError('overview must be of type str')
    self._overview = overview

  @property
  def usage(self) -> str:
    return self._usage

  @usage.setter
  def usage(self, usage: str):
    if usage is None:
      del self.usage
      return
    if not isinstance(usage, str):
      raise TypeError('usage must be of type str')
    self._usage = usage

  @property
  def rendered_usage(self) -> str:
    return self._rendered_usage

  @rendered_usage.setter
  def rendered_usage(self, rendered_usage: str):
    if rendered_usage is None:
      del self.rendered_usage
      return
    if not isinstance(rendered_usage, str):
      raise TypeError('rendered_usage must be of type str')
    self._rendered_usage = rendered_usage

  @property
  def text_representation(self) -> str:
    return self._text_representation or ""

  @text_representation.setter
  def text_representation(self, text_representation: Optional[str]):
    if text_representation is None:
      del self.text_representation
      return
    if not isinstance(text_representation, str):
      raise TypeError('text_representation must be of type str')
    self._text_representation = text_representation

  @property
  def source_url(self) -> str:
    return self._source_url or ""

  @source_url.setter
  def source_url(self, source_url: Optional[str]):
    if source_url is None:
      del self.source_url
      return
    if not isinstance(source_url, str):
      raise TypeError('source_url must be of type str')
    self._source_url = source_url

  @property
  def version_number(self) -> int:
    return self._version_number

  @version_number.setter
  def version_number(self, version_number: int):
    if version_number is None:
      del self.version_number
      return
    if not isinstance(version_number, int):
      raise TypeError('version_number must be of type int')
    self._version_number = version_number

  @property
  def framework(self) -> 'ModelFramework':
    r"""
    TODO(http://b/253694274): Add support for variation tables when that's
    finished taking shape
    """
    return self._framework

  @framework.setter
  def framework(self, framework: 'ModelFramework'):
    if framework is None:
      del self.framework
      return
    if not isinstance(framework, ModelFramework):
      raise TypeError('framework must be of type ModelFramework')
    self._framework = framework

  @property
  def version_notes(self) -> str:
    return self._version_notes

  @version_notes.setter
  def version_notes(self, version_notes: str):
    if version_notes is None:
      del self.version_notes
      return
    if not isinstance(version_notes, str):
      raise TypeError('version_notes must be of type str')
    self._version_notes = version_notes

  @property
  def download_url(self) -> str:
    return self._download_url or ""

  @download_url.setter
  def download_url(self, download_url: Optional[str]):
    if download_url is None:
      del self.download_url
      return
    if not isinstance(download_url, str):
      raise TypeError('download_url must be of type str')
    self._download_url = download_url

  @property
  def databundle_id(self) -> int:
    return self._databundle_id

  @databundle_id.setter
  def databundle_id(self, databundle_id: int):
    if databundle_id is None:
      del self.databundle_id
      return
    if not isinstance(databundle_id, int):
      raise TypeError('databundle_id must be of type int')
    self._databundle_id = databundle_id

  @property
  def databundle_version_id(self) -> int:
    return self._databundle_version_id

  @databundle_version_id.setter
  def databundle_version_id(self, databundle_version_id: int):
    if databundle_version_id is None:
      del self.databundle_version_id
      return
    if not isinstance(databundle_version_id, int):
      raise TypeError('databundle_version_id must be of type int')
    self._databundle_version_id = databundle_version_id

  @property
  def databundle_version_type(self) -> 'DatabundleVersionType':
    return self._databundle_version_type

  @databundle_version_type.setter
  def databundle_version_type(self, databundle_version_type: 'DatabundleVersionType'):
    if databundle_version_type is None:
      del self.databundle_version_type
      return
    if not isinstance(databundle_version_type, DatabundleVersionType):
      raise TypeError('databundle_version_type must be of type DatabundleVersionType')
    self._databundle_version_type = databundle_version_type

  @property
  def firestore_path(self) -> str:
    return self._firestore_path

  @firestore_path.setter
  def firestore_path(self, firestore_path: str):
    if firestore_path is None:
      del self.firestore_path
      return
    if not isinstance(firestore_path, str):
      raise TypeError('firestore_path must be of type str')
    self._firestore_path = firestore_path

  @property
  def status(self) -> 'DatabundleVersionStatus':
    return self._status

  @status.setter
  def status(self, status: 'DatabundleVersionStatus'):
    if status is None:
      del self.status
      return
    if not isinstance(status, DatabundleVersionStatus):
      raise TypeError('status must be of type DatabundleVersionStatus')
    self._status = status

  @property
  def creation_status(self) -> Optional['DatabundleVersionCreationStatus']:
    return self._creation_status or None

  @creation_status.setter
  def creation_status(self, creation_status: Optional[Optional['DatabundleVersionCreationStatus']]):
    if creation_status is None:
      del self.creation_status
      return
    if not isinstance(creation_status, DatabundleVersionCreationStatus):
      raise TypeError('creation_status must be of type DatabundleVersionCreationStatus')
    self._creation_status = creation_status

  @property
  def error_message(self) -> str:
    return self._error_message or ""

  @error_message.setter
  def error_message(self, error_message: Optional[str]):
    if error_message is None:
      del self.error_message
      return
    if not isinstance(error_message, str):
      raise TypeError('error_message must be of type str')
    self._error_message = error_message

  @property
  def last_version_id(self) -> int:
    """Version ID associated with the most up-to-date version of the ModelInstance"""
    return self._last_version_id or 0

  @last_version_id.setter
  def last_version_id(self, last_version_id: Optional[int]):
    if last_version_id is None:
      del self.last_version_id
      return
    if not isinstance(last_version_id, int):
      raise TypeError('last_version_id must be of type int')
    self._last_version_id = last_version_id

  @property
  def source_organization(self) -> Optional['Owner']:
    return self._source_organization or None

  @source_organization.setter
  def source_organization(self, source_organization: Optional[Optional['Owner']]):
    if source_organization is None:
      del self.source_organization
      return
    if not isinstance(source_organization, Owner):
      raise TypeError('source_organization must be of type Owner')
    self._source_organization = source_organization

  @property
  def training_data(self) -> Optional[List[str]]:
    return self._training_data

  @training_data.setter
  def training_data(self, training_data: Optional[List[str]]):
    if training_data is None:
      del self.training_data
      return
    if not isinstance(training_data, list):
      raise TypeError('training_data must be of type list')
    if not all([isinstance(t, str) for t in training_data]):
      raise TypeError('training_data must contain only items of type str')
    self._training_data = training_data

  @property
  def metrics(self) -> str:
    return self._metrics or ""

  @metrics.setter
  def metrics(self, metrics: Optional[str]):
    if metrics is None:
      del self.metrics
      return
    if not isinstance(metrics, str):
      raise TypeError('metrics must be of type str')
    self._metrics = metrics

  @property
  def license_post(self) -> Optional['LicensePost']:
    return self._license_post or None

  @license_post.setter
  def license_post(self, license_post: Optional[Optional['LicensePost']]):
    if license_post is None:
      del self.license_post
      return
    if not isinstance(license_post, LicensePost):
      raise TypeError('license_post must be of type LicensePost')
    self._license_post = license_post

  @property
  def license(self) -> Optional['License']:
    return self._license or None

  @license.setter
  def license(self, license: Optional[Optional['License']]):
    if license is None:
      del self.license
      return
    if not isinstance(license, License):
      raise TypeError('license must be of type License')
    self._license = license

  @property
  def can_use(self) -> bool:
    return self._can_use or False

  @can_use.setter
  def can_use(self, can_use: Optional[bool]):
    if can_use is None:
      del self.can_use
      return
    if not isinstance(can_use, bool):
      raise TypeError('can_use must be of type bool')
    self._can_use = can_use

  @property
  def uncompressed_storage_uri(self) -> str:
    return self._uncompressed_storage_uri or ""

  @uncompressed_storage_uri.setter
  def uncompressed_storage_uri(self, uncompressed_storage_uri: Optional[str]):
    if uncompressed_storage_uri is None:
      del self.uncompressed_storage_uri
      return
    if not isinstance(uncompressed_storage_uri, str):
      raise TypeError('uncompressed_storage_uri must be of type str')
    self._uncompressed_storage_uri = uncompressed_storage_uri

  @property
  def model_instance_type(self) -> 'ModelInstanceType':
    return self._model_instance_type or ModelInstanceType.MODEL_INSTANCE_TYPE_UNSPECIFIED

  @model_instance_type.setter
  def model_instance_type(self, model_instance_type: Optional['ModelInstanceType']):
    if model_instance_type is None:
      del self.model_instance_type
      return
    if not isinstance(model_instance_type, ModelInstanceType):
      raise TypeError('model_instance_type must be of type ModelInstanceType')
    self._model_instance_type = model_instance_type

  @property
  def base_model_instance_id(self) -> int:
    return self._base_model_instance_id or 0

  @base_model_instance_id.setter
  def base_model_instance_id(self, base_model_instance_id: Optional[int]):
    if base_model_instance_id is None:
      del self.base_model_instance_id
      return
    if not isinstance(base_model_instance_id, int):
      raise TypeError('base_model_instance_id must be of type int')
    self._base_model_instance_id = base_model_instance_id

  @property
  def base_model_instance_information(self) -> Optional['BaseModelInstanceInformation']:
    return self._base_model_instance_information or None

  @base_model_instance_information.setter
  def base_model_instance_information(self, base_model_instance_information: Optional[Optional['BaseModelInstanceInformation']]):
    if base_model_instance_information is None:
      del self.base_model_instance_information
      return
    if not isinstance(base_model_instance_information, BaseModelInstanceInformation):
      raise TypeError('base_model_instance_information must be of type BaseModelInstanceInformation')
    self._base_model_instance_information = base_model_instance_information

  @property
  def external_base_model_url(self) -> str:
    return self._external_base_model_url or ""

  @external_base_model_url.setter
  def external_base_model_url(self, external_base_model_url: Optional[str]):
    if external_base_model_url is None:
      del self.external_base_model_url
      return
    if not isinstance(external_base_model_url, str):
      raise TypeError('external_base_model_url must be of type str')
    self._external_base_model_url = external_base_model_url

  @property
  def download_summary(self) -> Optional['ModelInstanceDownloadSummary']:
    return self._download_summary or None

  @download_summary.setter
  def download_summary(self, download_summary: Optional[Optional['ModelInstanceDownloadSummary']]):
    if download_summary is None:
      del self.download_summary
      return
    if not isinstance(download_summary, ModelInstanceDownloadSummary):
      raise TypeError('download_summary must be of type ModelInstanceDownloadSummary')
    self._download_summary = download_summary

  @property
  def total_uncompressed_bytes(self) -> int:
    return self._total_uncompressed_bytes or 0

  @total_uncompressed_bytes.setter
  def total_uncompressed_bytes(self, total_uncompressed_bytes: Optional[int]):
    if total_uncompressed_bytes is None:
      del self.total_uncompressed_bytes
      return
    if not isinstance(total_uncompressed_bytes, int):
      raise TypeError('total_uncompressed_bytes must be of type int')
    self._total_uncompressed_bytes = total_uncompressed_bytes

  @property
  def sigstore_state(self) -> 'SigstoreState':
    return self._sigstore_state or SigstoreState.SIGSTORE_STATE_UNSPECIFIED

  @sigstore_state.setter
  def sigstore_state(self, sigstore_state: Optional['SigstoreState']):
    if sigstore_state is None:
      del self.sigstore_state
      return
    if not isinstance(sigstore_state, SigstoreState):
      raise TypeError('sigstore_state must be of type SigstoreState')
    self._sigstore_state = sigstore_state

  @property
  def created_by_kernel_id(self) -> int:
    return self._created_by_kernel_id or 0

  @created_by_kernel_id.setter
  def created_by_kernel_id(self, created_by_kernel_id: Optional[int]):
    if created_by_kernel_id is None:
      del self.created_by_kernel_id
      return
    if not isinstance(created_by_kernel_id, int):
      raise TypeError('created_by_kernel_id must be of type int')
    self._created_by_kernel_id = created_by_kernel_id

  @property
  def creator_user_id(self) -> int:
    return self._creator_user_id or 0

  @creator_user_id.setter
  def creator_user_id(self, creator_user_id: Optional[int]):
    if creator_user_id is None:
      del self.creator_user_id
      return
    if not isinstance(creator_user_id, int):
      raise TypeError('creator_user_id must be of type int')
    self._creator_user_id = creator_user_id

  @property
  def attestation_kernel_url(self) -> str:
    return self._attestation_kernel_url or ""

  @attestation_kernel_url.setter
  def attestation_kernel_url(self, attestation_kernel_url: Optional[str]):
    if attestation_kernel_url is None:
      del self.attestation_kernel_url
      return
    if not isinstance(attestation_kernel_url, str):
      raise TypeError('attestation_kernel_url must be of type str')
    self._attestation_kernel_url = attestation_kernel_url


class ModelInstanceDownloadSummary(KaggleObject):
  r"""
  Attributes:
    total_downloads (float)
    download_series_points (ModelActivityTimeSeriesPoint)
    model_instance_id (int)
  """

  def __init__(self):
    self._total_downloads = 0.0
    self._download_series_points = []
    self._model_instance_id = 0
    self._freeze()

  @property
  def total_downloads(self) -> float:
    return self._total_downloads

  @total_downloads.setter
  def total_downloads(self, total_downloads: float):
    if total_downloads is None:
      del self.total_downloads
      return
    if not isinstance(total_downloads, float):
      raise TypeError('total_downloads must be of type float')
    self._total_downloads = total_downloads

  @property
  def download_series_points(self) -> Optional[List[Optional['ModelActivityTimeSeriesPoint']]]:
    return self._download_series_points

  @download_series_points.setter
  def download_series_points(self, download_series_points: Optional[List[Optional['ModelActivityTimeSeriesPoint']]]):
    if download_series_points is None:
      del self.download_series_points
      return
    if not isinstance(download_series_points, list):
      raise TypeError('download_series_points must be of type list')
    if not all([isinstance(t, ModelActivityTimeSeriesPoint) for t in download_series_points]):
      raise TypeError('download_series_points must contain only items of type ModelActivityTimeSeriesPoint')
    self._download_series_points = download_series_points

  @property
  def model_instance_id(self) -> int:
    return self._model_instance_id

  @model_instance_id.setter
  def model_instance_id(self, model_instance_id: int):
    if model_instance_id is None:
      del self.model_instance_id
      return
    if not isinstance(model_instance_id, int):
      raise TypeError('model_instance_id must be of type int')
    self._model_instance_id = model_instance_id


class ModelInstanceVersionList(KaggleObject):
  r"""
  Attributes:
    versions (ModelInstanceVersion)
  """

  def __init__(self):
    self._versions = []
    self._freeze()

  @property
  def versions(self) -> Optional[List[Optional['ModelInstanceVersion']]]:
    return self._versions

  @versions.setter
  def versions(self, versions: Optional[List[Optional['ModelInstanceVersion']]]):
    if versions is None:
      del self.versions
      return
    if not isinstance(versions, list):
      raise TypeError('versions must be of type list')
    if not all([isinstance(t, ModelInstanceVersion) for t in versions]):
      raise TypeError('versions must contain only items of type ModelInstanceVersion')
    self._versions = versions


class ModelLink(KaggleObject):
  r"""
  Attributes:
    type (ModelVersionLinkType)
    url (str)
  """

  def __init__(self):
    self._type = ModelVersionLinkType.MODEL_VERSION_LINK_TYPE_UNSPECIFIED
    self._url = ""
    self._freeze()

  @property
  def type(self) -> 'ModelVersionLinkType':
    return self._type

  @type.setter
  def type(self, type: 'ModelVersionLinkType'):
    if type is None:
      del self.type
      return
    if not isinstance(type, ModelVersionLinkType):
      raise TypeError('type must be of type ModelVersionLinkType')
    self._type = type

  @property
  def url(self) -> str:
    return self._url

  @url.setter
  def url(self, url: str):
    if url is None:
      del self.url
      return
    if not isinstance(url, str):
      raise TypeError('url must be of type str')
    self._url = url


class Owner(KaggleObject):
  r"""
  Based off Datasets OwnerDto as the permission model is the same
  Separate message since Models don't have max_file_size_bytes.
  Consider renaming more generically to apply to Users/Orgs
  interchangeably without a strict concept of ownership

  Attributes:
    id (int)
    image_url (str)
    is_organization (bool)
    name (str)
    profile_url (str)
    slug (str)
    user_tier (UserAchievementTier)
    user_progression_opt_out (bool)
      Whether or not the owner is progression opted-out (only for user owners).
    allow_model_gating (bool)
  """

  def __init__(self):
    self._id = 0
    self._image_url = None
    self._is_organization = False
    self._name = ""
    self._profile_url = None
    self._slug = ""
    self._user_tier = UserAchievementTier.NOVICE
    self._user_progression_opt_out = None
    self._allow_model_gating = None
    self._freeze()

  @property
  def id(self) -> int:
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id

  @property
  def image_url(self) -> str:
    return self._image_url or ""

  @image_url.setter
  def image_url(self, image_url: Optional[str]):
    if image_url is None:
      del self.image_url
      return
    if not isinstance(image_url, str):
      raise TypeError('image_url must be of type str')
    self._image_url = image_url

  @property
  def is_organization(self) -> bool:
    return self._is_organization

  @is_organization.setter
  def is_organization(self, is_organization: bool):
    if is_organization is None:
      del self.is_organization
      return
    if not isinstance(is_organization, bool):
      raise TypeError('is_organization must be of type bool')
    self._is_organization = is_organization

  @property
  def name(self) -> str:
    return self._name

  @name.setter
  def name(self, name: str):
    if name is None:
      del self.name
      return
    if not isinstance(name, str):
      raise TypeError('name must be of type str')
    self._name = name

  @property
  def profile_url(self) -> str:
    return self._profile_url or ""

  @profile_url.setter
  def profile_url(self, profile_url: Optional[str]):
    if profile_url is None:
      del self.profile_url
      return
    if not isinstance(profile_url, str):
      raise TypeError('profile_url must be of type str')
    self._profile_url = profile_url

  @property
  def slug(self) -> str:
    return self._slug

  @slug.setter
  def slug(self, slug: str):
    if slug is None:
      del self.slug
      return
    if not isinstance(slug, str):
      raise TypeError('slug must be of type str')
    self._slug = slug

  @property
  def user_tier(self) -> 'UserAchievementTier':
    return self._user_tier

  @user_tier.setter
  def user_tier(self, user_tier: 'UserAchievementTier'):
    if user_tier is None:
      del self.user_tier
      return
    if not isinstance(user_tier, UserAchievementTier):
      raise TypeError('user_tier must be of type UserAchievementTier')
    self._user_tier = user_tier

  @property
  def user_progression_opt_out(self) -> bool:
    """Whether or not the owner is progression opted-out (only for user owners)."""
    return self._user_progression_opt_out or False

  @user_progression_opt_out.setter
  def user_progression_opt_out(self, user_progression_opt_out: Optional[bool]):
    if user_progression_opt_out is None:
      del self.user_progression_opt_out
      return
    if not isinstance(user_progression_opt_out, bool):
      raise TypeError('user_progression_opt_out must be of type bool')
    self._user_progression_opt_out = user_progression_opt_out

  @property
  def allow_model_gating(self) -> bool:
    return self._allow_model_gating or False

  @allow_model_gating.setter
  def allow_model_gating(self, allow_model_gating: Optional[bool]):
    if allow_model_gating is None:
      del self.allow_model_gating
      return
    if not isinstance(allow_model_gating, bool):
      raise TypeError('allow_model_gating must be of type bool')
    self._allow_model_gating = allow_model_gating


class LicensePost(KaggleObject):
  r"""
  Attributes:
    id (int)
    content (str)
  """

  def __init__(self):
    self._id = 0
    self._content = ""
    self._freeze()

  @property
  def id(self) -> int:
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id

  @property
  def content(self) -> str:
    return self._content

  @content.setter
  def content(self, content: str):
    if content is None:
      del self.content
      return
    if not isinstance(content, str):
      raise TypeError('content must be of type str')
    self._content = content


class ModelActivityTimeSeriesPoint(KaggleObject):
  r"""
  Attributes:
    date (datetime)
    count (int)
  """

  def __init__(self):
    self._date = None
    self._count = 0
    self._freeze()

  @property
  def date(self) -> datetime:
    return self._date

  @date.setter
  def date(self, date: datetime):
    if date is None:
      del self.date
      return
    if not isinstance(date, datetime):
      raise TypeError('date must be of type datetime')
    self._date = date

  @property
  def count(self) -> int:
    return self._count

  @count.setter
  def count(self, count: int):
    if count is None:
      del self.count
      return
    if not isinstance(count, int):
      raise TypeError('count must be of type int')
    self._count = count


class ModelInstanceVersion(KaggleObject):
  r"""
  Attributes:
    id (int)
    framework (ModelFramework)
    is_tfhub_model (bool)
    url (str)
    variation_slug (str)
    version_number (int)
    model_title (str)
    thumbnail_url (str)
    is_private (bool)
    sigstore_state (SigstoreState)
  """

  def __init__(self):
    self._id = 0
    self._framework = ModelFramework.MODEL_FRAMEWORK_UNSPECIFIED
    self._is_tfhub_model = False
    self._url = ""
    self._variation_slug = ""
    self._version_number = 0
    self._model_title = ""
    self._thumbnail_url = ""
    self._is_private = False
    self._sigstore_state = SigstoreState.SIGSTORE_STATE_UNSPECIFIED
    self._freeze()

  @property
  def id(self) -> int:
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id

  @property
  def framework(self) -> 'ModelFramework':
    return self._framework

  @framework.setter
  def framework(self, framework: 'ModelFramework'):
    if framework is None:
      del self.framework
      return
    if not isinstance(framework, ModelFramework):
      raise TypeError('framework must be of type ModelFramework')
    self._framework = framework

  @property
  def is_tfhub_model(self) -> bool:
    return self._is_tfhub_model

  @is_tfhub_model.setter
  def is_tfhub_model(self, is_tfhub_model: bool):
    if is_tfhub_model is None:
      del self.is_tfhub_model
      return
    if not isinstance(is_tfhub_model, bool):
      raise TypeError('is_tfhub_model must be of type bool')
    self._is_tfhub_model = is_tfhub_model

  @property
  def url(self) -> str:
    return self._url

  @url.setter
  def url(self, url: str):
    if url is None:
      del self.url
      return
    if not isinstance(url, str):
      raise TypeError('url must be of type str')
    self._url = url

  @property
  def variation_slug(self) -> str:
    return self._variation_slug

  @variation_slug.setter
  def variation_slug(self, variation_slug: str):
    if variation_slug is None:
      del self.variation_slug
      return
    if not isinstance(variation_slug, str):
      raise TypeError('variation_slug must be of type str')
    self._variation_slug = variation_slug

  @property
  def version_number(self) -> int:
    return self._version_number

  @version_number.setter
  def version_number(self, version_number: int):
    if version_number is None:
      del self.version_number
      return
    if not isinstance(version_number, int):
      raise TypeError('version_number must be of type int')
    self._version_number = version_number

  @property
  def model_title(self) -> str:
    return self._model_title

  @model_title.setter
  def model_title(self, model_title: str):
    if model_title is None:
      del self.model_title
      return
    if not isinstance(model_title, str):
      raise TypeError('model_title must be of type str')
    self._model_title = model_title

  @property
  def thumbnail_url(self) -> str:
    return self._thumbnail_url

  @thumbnail_url.setter
  def thumbnail_url(self, thumbnail_url: str):
    if thumbnail_url is None:
      del self.thumbnail_url
      return
    if not isinstance(thumbnail_url, str):
      raise TypeError('thumbnail_url must be of type str')
    self._thumbnail_url = thumbnail_url

  @property
  def is_private(self) -> bool:
    return self._is_private

  @is_private.setter
  def is_private(self, is_private: bool):
    if is_private is None:
      del self.is_private
      return
    if not isinstance(is_private, bool):
      raise TypeError('is_private must be of type bool')
    self._is_private = is_private

  @property
  def sigstore_state(self) -> 'SigstoreState':
    return self._sigstore_state

  @sigstore_state.setter
  def sigstore_state(self, sigstore_state: 'SigstoreState'):
    if sigstore_state is None:
      del self.sigstore_state
      return
    if not isinstance(sigstore_state, SigstoreState):
      raise TypeError('sigstore_state must be of type SigstoreState')
    self._sigstore_state = sigstore_state


BaseModelInstanceInformation._fields = [
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
  FieldMetadata("owner", "owner", "_owner", Owner, None, KaggleObjectSerializer()),
  FieldMetadata("modelSlug", "model_slug", "_model_slug", str, "", PredefinedSerializer()),
  FieldMetadata("instanceSlug", "instance_slug", "_instance_slug", str, "", PredefinedSerializer()),
  FieldMetadata("framework", "framework", "_framework", ModelFramework, ModelFramework.MODEL_FRAMEWORK_UNSPECIFIED, EnumSerializer()),
]

ModelInstance._fields = [
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
  FieldMetadata("ownerSlug", "owner_slug", "_owner_slug", str, "", PredefinedSerializer()),
  FieldMetadata("modelSlug", "model_slug", "_model_slug", str, "", PredefinedSerializer()),
  FieldMetadata("modelId", "model_id", "_model_id", int, 0, PredefinedSerializer()),
  FieldMetadata("slug", "slug", "_slug", str, "", PredefinedSerializer()),
  FieldMetadata("versionId", "version_id", "_version_id", int, 0, PredefinedSerializer()),
  FieldMetadata("fineTunable", "fine_tunable", "_fine_tunable", bool, None, PredefinedSerializer(), optional=True),
  FieldMetadata("overview", "overview", "_overview", str, "", PredefinedSerializer()),
  FieldMetadata("usage", "usage", "_usage", str, "", PredefinedSerializer()),
  FieldMetadata("renderedUsage", "rendered_usage", "_rendered_usage", str, "", PredefinedSerializer()),
  FieldMetadata("textRepresentation", "text_representation", "_text_representation", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("sourceUrl", "source_url", "_source_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("versionNumber", "version_number", "_version_number", int, 0, PredefinedSerializer()),
  FieldMetadata("framework", "framework", "_framework", ModelFramework, ModelFramework.MODEL_FRAMEWORK_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("versionNotes", "version_notes", "_version_notes", str, "", PredefinedSerializer()),
  FieldMetadata("downloadUrl", "download_url", "_download_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("databundleId", "databundle_id", "_databundle_id", int, 0, PredefinedSerializer()),
  FieldMetadata("databundleVersionId", "databundle_version_id", "_databundle_version_id", int, 0, PredefinedSerializer()),
  FieldMetadata("databundleVersionType", "databundle_version_type", "_databundle_version_type", DatabundleVersionType, DatabundleVersionType.DATABUNDLE_VERSION_TYPE_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("firestorePath", "firestore_path", "_firestore_path", str, "", PredefinedSerializer()),
  FieldMetadata("status", "status", "_status", DatabundleVersionStatus, DatabundleVersionStatus.NOT_YET_PERSISTED, EnumSerializer()),
  FieldMetadata("creationStatus", "creation_status", "_creation_status", DatabundleVersionCreationStatus, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("errorMessage", "error_message", "_error_message", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("lastVersionId", "last_version_id", "_last_version_id", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("sourceOrganization", "source_organization", "_source_organization", Owner, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("trainingData", "training_data", "_training_data", str, [], ListSerializer(PredefinedSerializer())),
  FieldMetadata("metrics", "metrics", "_metrics", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("licensePost", "license_post", "_license_post", LicensePost, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("license", "license", "_license", License, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("canUse", "can_use", "_can_use", bool, None, PredefinedSerializer(), optional=True),
  FieldMetadata("uncompressedStorageUri", "uncompressed_storage_uri", "_uncompressed_storage_uri", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("modelInstanceType", "model_instance_type", "_model_instance_type", ModelInstanceType, None, EnumSerializer(), optional=True),
  FieldMetadata("baseModelInstanceId", "base_model_instance_id", "_base_model_instance_id", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("baseModelInstanceInformation", "base_model_instance_information", "_base_model_instance_information", BaseModelInstanceInformation, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("externalBaseModelUrl", "external_base_model_url", "_external_base_model_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("downloadSummary", "download_summary", "_download_summary", ModelInstanceDownloadSummary, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("totalUncompressedBytes", "total_uncompressed_bytes", "_total_uncompressed_bytes", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("sigstoreState", "sigstore_state", "_sigstore_state", SigstoreState, None, EnumSerializer(), optional=True),
  FieldMetadata("createdByKernelId", "created_by_kernel_id", "_created_by_kernel_id", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("creatorUserId", "creator_user_id", "_creator_user_id", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("attestationKernelUrl", "attestation_kernel_url", "_attestation_kernel_url", str, None, PredefinedSerializer(), optional=True),
]

ModelInstanceDownloadSummary._fields = [
  FieldMetadata("totalDownloads", "total_downloads", "_total_downloads", float, 0.0, PredefinedSerializer()),
  FieldMetadata("downloadSeriesPoints", "download_series_points", "_download_series_points", ModelActivityTimeSeriesPoint, [], ListSerializer(KaggleObjectSerializer())),
  FieldMetadata("modelInstanceId", "model_instance_id", "_model_instance_id", int, 0, PredefinedSerializer()),
]

ModelInstanceVersionList._fields = [
  FieldMetadata("versions", "versions", "_versions", ModelInstanceVersion, [], ListSerializer(KaggleObjectSerializer())),
]

ModelLink._fields = [
  FieldMetadata("type", "type", "_type", ModelVersionLinkType, ModelVersionLinkType.MODEL_VERSION_LINK_TYPE_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("url", "url", "_url", str, "", PredefinedSerializer()),
]

Owner._fields = [
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
  FieldMetadata("imageUrl", "image_url", "_image_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("isOrganization", "is_organization", "_is_organization", bool, False, PredefinedSerializer()),
  FieldMetadata("name", "name", "_name", str, "", PredefinedSerializer()),
  FieldMetadata("profileUrl", "profile_url", "_profile_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("slug", "slug", "_slug", str, "", PredefinedSerializer()),
  FieldMetadata("userTier", "user_tier", "_user_tier", UserAchievementTier, UserAchievementTier.NOVICE, EnumSerializer()),
  FieldMetadata("userProgressionOptOut", "user_progression_opt_out", "_user_progression_opt_out", bool, None, PredefinedSerializer(), optional=True),
  FieldMetadata("allowModelGating", "allow_model_gating", "_allow_model_gating", bool, None, PredefinedSerializer(), optional=True),
]

LicensePost._fields = [
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
  FieldMetadata("content", "content", "_content", str, "", PredefinedSerializer()),
]

ModelActivityTimeSeriesPoint._fields = [
  FieldMetadata("date", "date", "_date", datetime, None, DateTimeSerializer()),
  FieldMetadata("count", "count", "_count", int, 0, PredefinedSerializer()),
]

ModelInstanceVersion._fields = [
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
  FieldMetadata("framework", "framework", "_framework", ModelFramework, ModelFramework.MODEL_FRAMEWORK_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("isTfhubModel", "is_tfhub_model", "_is_tfhub_model", bool, False, PredefinedSerializer()),
  FieldMetadata("url", "url", "_url", str, "", PredefinedSerializer()),
  FieldMetadata("variationSlug", "variation_slug", "_variation_slug", str, "", PredefinedSerializer()),
  FieldMetadata("versionNumber", "version_number", "_version_number", int, 0, PredefinedSerializer()),
  FieldMetadata("modelTitle", "model_title", "_model_title", str, "", PredefinedSerializer()),
  FieldMetadata("thumbnailUrl", "thumbnail_url", "_thumbnail_url", str, "", PredefinedSerializer()),
  FieldMetadata("isPrivate", "is_private", "_is_private", bool, False, PredefinedSerializer()),
  FieldMetadata("sigstoreState", "sigstore_state", "_sigstore_state", SigstoreState, SigstoreState.SIGSTORE_STATE_UNSPECIFIED, EnumSerializer()),
]




from datetime import datetime
from kagglesdk.community.types.content_enums import ContentState
from kagglesdk.discussions.types.writeup_enums import WriteUpType
from kagglesdk.kaggle_object import *
from kagglesdk.users.types.user_avatar import UserAvatar
from typing import Optional, List

class WriteUpItemInfo(KaggleObject):
  r"""
  Attributes:
    type (WriteUpType)
      Type of WriteUp
    subtitle (str)
      Subtitle of WriteUp
    collaborators (UserAvatar)
      List of WriteUp collaborators
    competition_info (WriteUpCompetitionInfo)
      Competition metadata associated with WriteUp
    content_state (ContentState)
      Content State of WriteUp
    team_name (str)
      Name of the team that owns the WriteUp
    id (int)
      Id of the WriteUp
  """

  def __init__(self):
    self._type = WriteUpType.WRITE_UP_TYPE_UNSPECIFIED
    self._subtitle = None
    self._collaborators = []
    self._competition_info = None
    self._content_state = ContentState.CONTENT_STATE_UNSPECIFIED
    self._team_name = None
    self._id = 0
    self._freeze()

  @property
  def type(self) -> 'WriteUpType':
    """Type of WriteUp"""
    return self._type

  @type.setter
  def type(self, type: 'WriteUpType'):
    if type is None:
      del self.type
      return
    if not isinstance(type, WriteUpType):
      raise TypeError('type must be of type WriteUpType')
    self._type = type

  @property
  def subtitle(self) -> str:
    """Subtitle of WriteUp"""
    return self._subtitle or ""

  @subtitle.setter
  def subtitle(self, subtitle: Optional[str]):
    if subtitle is None:
      del self.subtitle
      return
    if not isinstance(subtitle, str):
      raise TypeError('subtitle must be of type str')
    self._subtitle = subtitle

  @property
  def collaborators(self) -> Optional[List[Optional['UserAvatar']]]:
    """List of WriteUp collaborators"""
    return self._collaborators

  @collaborators.setter
  def collaborators(self, collaborators: Optional[List[Optional['UserAvatar']]]):
    if collaborators is None:
      del self.collaborators
      return
    if not isinstance(collaborators, list):
      raise TypeError('collaborators must be of type list')
    if not all([isinstance(t, UserAvatar) for t in collaborators]):
      raise TypeError('collaborators must contain only items of type UserAvatar')
    self._collaborators = collaborators

  @property
  def competition_info(self) -> Optional['WriteUpCompetitionInfo']:
    """Competition metadata associated with WriteUp"""
    return self._competition_info or None

  @competition_info.setter
  def competition_info(self, competition_info: Optional[Optional['WriteUpCompetitionInfo']]):
    if competition_info is None:
      del self.competition_info
      return
    if not isinstance(competition_info, WriteUpCompetitionInfo):
      raise TypeError('competition_info must be of type WriteUpCompetitionInfo')
    self._competition_info = competition_info

  @property
  def content_state(self) -> 'ContentState':
    """Content State of WriteUp"""
    return self._content_state

  @content_state.setter
  def content_state(self, content_state: 'ContentState'):
    if content_state is None:
      del self.content_state
      return
    if not isinstance(content_state, ContentState):
      raise TypeError('content_state must be of type ContentState')
    self._content_state = content_state

  @property
  def team_name(self) -> str:
    """Name of the team that owns the WriteUp"""
    return self._team_name or ""

  @team_name.setter
  def team_name(self, team_name: Optional[str]):
    if team_name is None:
      del self.team_name
      return
    if not isinstance(team_name, str):
      raise TypeError('team_name must be of type str')
    self._team_name = team_name

  @property
  def id(self) -> int:
    """Id of the WriteUp"""
    return self._id

  @id.setter
  def id(self, id: int):
    if id is None:
      del self.id
      return
    if not isinstance(id, int):
      raise TypeError('id must be of type int')
    self._id = id


class WriteUpCompetitionInfo(KaggleObject):
  r"""
  Attributes:
    competition_title (str)
      Title of the Competition or Hackathon
    competition_url (str)
      Url of the Competition or Hackathon
    deadline (datetime)
      Deadline of the Competition or Hackathon
    write_up_leaderboard_rank (int)
      Rank of Competition Solution WriteUp on Leaderboard
    leaderboard_url (str)
      Leaderboard Tab Url for Competition
    winners_url (str)
      Winners Tab Url of Hackathon
    is_hackathon_winner (bool)
      Boolean to tell if user's WriteUp is winner of Hackathon
    competition_id (int)
      Id of the Competition
  """

  def __init__(self):
    self._competition_title = ""
    self._competition_url = ""
    self._deadline = None
    self._write_up_leaderboard_rank = None
    self._leaderboard_url = None
    self._winners_url = None
    self._is_hackathon_winner = None
    self._competition_id = 0
    self._freeze()

  @property
  def competition_title(self) -> str:
    """Title of the Competition or Hackathon"""
    return self._competition_title

  @competition_title.setter
  def competition_title(self, competition_title: str):
    if competition_title is None:
      del self.competition_title
      return
    if not isinstance(competition_title, str):
      raise TypeError('competition_title must be of type str')
    self._competition_title = competition_title

  @property
  def competition_url(self) -> str:
    """Url of the Competition or Hackathon"""
    return self._competition_url

  @competition_url.setter
  def competition_url(self, competition_url: str):
    if competition_url is None:
      del self.competition_url
      return
    if not isinstance(competition_url, str):
      raise TypeError('competition_url must be of type str')
    self._competition_url = competition_url

  @property
  def deadline(self) -> datetime:
    """Deadline of the Competition or Hackathon"""
    return self._deadline

  @deadline.setter
  def deadline(self, deadline: datetime):
    if deadline is None:
      del self.deadline
      return
    if not isinstance(deadline, datetime):
      raise TypeError('deadline must be of type datetime')
    self._deadline = deadline

  @property
  def write_up_leaderboard_rank(self) -> int:
    """Rank of Competition Solution WriteUp on Leaderboard"""
    return self._write_up_leaderboard_rank or 0

  @write_up_leaderboard_rank.setter
  def write_up_leaderboard_rank(self, write_up_leaderboard_rank: Optional[int]):
    if write_up_leaderboard_rank is None:
      del self.write_up_leaderboard_rank
      return
    if not isinstance(write_up_leaderboard_rank, int):
      raise TypeError('write_up_leaderboard_rank must be of type int')
    self._write_up_leaderboard_rank = write_up_leaderboard_rank

  @property
  def leaderboard_url(self) -> str:
    """Leaderboard Tab Url for Competition"""
    return self._leaderboard_url or ""

  @leaderboard_url.setter
  def leaderboard_url(self, leaderboard_url: Optional[str]):
    if leaderboard_url is None:
      del self.leaderboard_url
      return
    if not isinstance(leaderboard_url, str):
      raise TypeError('leaderboard_url must be of type str')
    self._leaderboard_url = leaderboard_url

  @property
  def winners_url(self) -> str:
    """Winners Tab Url of Hackathon"""
    return self._winners_url or ""

  @winners_url.setter
  def winners_url(self, winners_url: Optional[str]):
    if winners_url is None:
      del self.winners_url
      return
    if not isinstance(winners_url, str):
      raise TypeError('winners_url must be of type str')
    self._winners_url = winners_url

  @property
  def is_hackathon_winner(self) -> bool:
    """Boolean to tell if user's WriteUp is winner of Hackathon"""
    return self._is_hackathon_winner or False

  @is_hackathon_winner.setter
  def is_hackathon_winner(self, is_hackathon_winner: Optional[bool]):
    if is_hackathon_winner is None:
      del self.is_hackathon_winner
      return
    if not isinstance(is_hackathon_winner, bool):
      raise TypeError('is_hackathon_winner must be of type bool')
    self._is_hackathon_winner = is_hackathon_winner

  @property
  def competition_id(self) -> int:
    """Id of the Competition"""
    return self._competition_id

  @competition_id.setter
  def competition_id(self, competition_id: int):
    if competition_id is None:
      del self.competition_id
      return
    if not isinstance(competition_id, int):
      raise TypeError('competition_id must be of type int')
    self._competition_id = competition_id


WriteUpItemInfo._fields = [
  FieldMetadata("type", "type", "_type", WriteUpType, WriteUpType.WRITE_UP_TYPE_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("subtitle", "subtitle", "_subtitle", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("collaborators", "collaborators", "_collaborators", UserAvatar, [], ListSerializer(KaggleObjectSerializer())),
  FieldMetadata("competitionInfo", "competition_info", "_competition_info", WriteUpCompetitionInfo, None, KaggleObjectSerializer(), optional=True),
  FieldMetadata("contentState", "content_state", "_content_state", ContentState, ContentState.CONTENT_STATE_UNSPECIFIED, EnumSerializer()),
  FieldMetadata("teamName", "team_name", "_team_name", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("id", "id", "_id", int, 0, PredefinedSerializer()),
]

WriteUpCompetitionInfo._fields = [
  FieldMetadata("competitionTitle", "competition_title", "_competition_title", str, "", PredefinedSerializer()),
  FieldMetadata("competitionUrl", "competition_url", "_competition_url", str, "", PredefinedSerializer()),
  FieldMetadata("deadline", "deadline", "_deadline", datetime, None, DateTimeSerializer()),
  FieldMetadata("writeUpLeaderboardRank", "write_up_leaderboard_rank", "_write_up_leaderboard_rank", int, None, PredefinedSerializer(), optional=True),
  FieldMetadata("leaderboardUrl", "leaderboard_url", "_leaderboard_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("winnersUrl", "winners_url", "_winners_url", str, None, PredefinedSerializer(), optional=True),
  FieldMetadata("isHackathonWinner", "is_hackathon_winner", "_is_hackathon_winner", bool, None, PredefinedSerializer(), optional=True),
  FieldMetadata("competitionId", "competition_id", "_competition_id", int, 0, PredefinedSerializer()),
]

